/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <stdio.h>

/*
 * Assumes 8 bits per byte.
 * LSB is bit number 0.
 */

typedef unsigned char Dx_ui8;
typedef signed char Dx_i8;

typedef struct {
  Dx_ui8 start_bitnum[8];
} Avail_info;

typedef struct byte_info {
  Dx_ui8 t_bits;			/* Total number of 0 bits. */
  Dx_ui8 r_bits;			/* Number of contiguous 0 bits from rhs */
  Dx_ui8 l_bits;			/* Number of contiguous 0 bits from lhs */
  Dx_ui8 max_bits;   	/* Maximum number of contiguous bits anywhere */
  Dx_i8  max_bits_start; /* Index of LSB where max. sequence starts (or -1) */
  Dx_i8  i_bit_index;    /* Index of an isolated bit (or LSB) (or -1) */
  Dx_ui8 c_bits[8];      /* Number of contiguous 0 bits (0-8) indexed by
						   starting bit number, with 16 added if the sequence
						   continues through to bit 7 */
} Byte_info;

Byte_info byte_info[256];

char *bitstring(int val);

int
main(int argc, char **argv)
{
  int bitnum, bytenum, mask, max, start_pos;
  Byte_info *bi;

  printf("/* This file was automatically generated by mkphysbits. */\n\n");
  printf("\nenum { BYTEINFO_C_BITS_COUNT = 0x0f };\n");
  printf("enum { BYTEINFO_C_BITS_ENDRUN = 0x10 }; \n");
  printf("\ntypedef struct {\n");
  printf("  Dx_ui8 t_bits;         /* Total number of 0 bits. */\n");
  printf("  Dx_ui8 r_bits;         /* Number of contiguous 0 bits from rhs */\n");
  printf("  Dx_ui8 l_bits;         /* Number of contiguous 0 bits from lhs */\n");
  printf("  Dx_ui8 max_bits;       /* Maximum number of contiguous bits anywhere */\n");
  printf("  Dx_i8  max_bits_start; /* Index of LSB where max. sequence starts */\n");
  printf("  Dx_i8  i_bit_index;    /* Index of an isolated bit (or LSB) */\n");
  printf("  Dx_i8  c_bits[8];      /* Number of contiguous 0 bits (0-8) indexed by\n");
  printf("                           starting bit number, with 16 added if the sequence\n");
  printf("                           continues through to bit 7 */\n");
  printf("} Byteinfo;\n");
  printf("\nstatic const Byteinfo byte_info[256] = {\n");

  /* Examine each bit pattern and build the table. */
  for (bytenum = 0; bytenum < 256; bytenum++) {
    bi = &byte_info[bytenum];
    bi->t_bits = bi->r_bits = bi->l_bits
		= bi->max_bits = 0;
    bi->max_bits_start = bi->i_bit_index = -1;

	/* Compute total 0 bits. */
    for (bitnum = 0; bitnum < 8; bitnum++) {
      if ((bytenum & (1 << bitnum)) == 0)
        bi->t_bits++;
	}

	/* Compute contiguous 0 bits from rhs. */
    for (bitnum = 0; bitnum < 8; bitnum++) {
      if ((bytenum & (1 << bitnum)) != 0)
		break;
      bi->r_bits++;
    }

	/* Compute contiguous 0 bits from lhs. */
    for (bitnum = 7; bitnum >= 0; bitnum--) {
      if ((bytenum & (1 << bitnum)) != 0)
		break;
      bi->l_bits++;
    }

	/* Compute max contiguous 0 bits anywhere. */
	max = 0;
	start_pos = 0;	/* Shuts up compiler. */
    for (bitnum = 0; bitnum < 8; bitnum++) {
      if ((bytenum & (1 << bitnum)) != 0) {
	    /* End of a sequence. */
		if (max > bi->max_bits) {
		  bi->max_bits = max;
		  bi->max_bits_start = start_pos;
		}
	    max = 0;
	  } else {
		if (max == 0) {
		  max = 1;
		  start_pos = bitnum;
	    }
		else
		  max++;
	  }
    }
	if (max > bi->max_bits) {
	  bi->max_bits = max;
	  bi->max_bits_start = start_pos;
    }

	/*
	 * Compute the index of an isolated bit (or LSB).
	 * The heuristic is to look for a a 0 bit that is surrounded by 1 bits.
	 * If there's no such bit, look for any 0 bit that's not within the
	 * maximum sequence of 0 bits.
	 * If there's still nothing, look for any 0 bit, starting with the LSB.
	 */
	if (bytenum == 0200)
		bi->i_bit_index = 7;
	else if (bytenum != 0377) {
		/* Look for a 0 bit with a 1 bit on either side. */
	    for (bitnum = 1; bitnum < 7; bitnum++) {
			if ((bytenum & (1 << (bitnum - 1))) != 0
					&& (bytenum & (1 << (bitnum + 0))) == 0
					&& (bytenum & (1 << (bitnum + 1))) != 0) {
				bi->i_bit_index = bitnum;
				break;
			}
		}
		if (bi->i_bit_index == -1) {
			/* Nothing found yet... */
			if ((bytenum & 1) == 0 && (bytenum & 2) != 0)
				bi->i_bit_index = 0;
			else if ((bytenum & 0200) == 0 && (bytenum & 0100) != 0)
				bi->i_bit_index = 7;
			else {
	    		for (bitnum = 0; bitnum < 8; bitnum++) {
					if (bitnum < bi->max_bits_start
						|| bitnum >= (bi->max_bits_start + bi->max_bits)) {
						bi->i_bit_index = bitnum;
						break;
					}
				}
	    		for (bitnum = 0; bitnum < 8; bitnum++) {
					if ((bytenum & (1 << bitnum)) == 0) {
						bi->i_bit_index = bitnum;
						break;
					}
				}
				if (bi->i_bit_index == -1) {
					fprintf(stderr, "i_bit_index bug!\n");
					exit(1);
				}
			}
		}
	}

	for (bitnum = 0; bitnum < 8; bitnum++)
	  bi->c_bits[bitnum] = 0;
	for (bitnum = 0; bitnum < 8; bitnum++) {
	  mask = (1 << bitnum);
	  for (start_pos = bitnum; start_pos < 8; start_pos++) {
		if (bytenum & mask)
		  break;
		bi->c_bits[bitnum]++;
		mask = mask << 1;
	  }
	  if (start_pos == 8)
		bi->c_bits[bitnum] += 16;
	}

	printf("  { %d, %d, %d, %d,%2d,%2d, ",
		   bi->t_bits, bi->r_bits, bi->l_bits,
		   bi->max_bits, bi->max_bits_start,
		   bi->i_bit_index);
	printf("{%2d,%2d,%2d,%2d,%2d,%2d,%2d,%2d}",
		   bi->c_bits[0], bi->c_bits[1], bi->c_bits[2], bi->c_bits[3],
		   bi->c_bits[4], bi->c_bits[5], bi->c_bits[6], bi->c_bits[7]);
	printf("}%s  /* %3d - %s */\n",
		   (bytenum != 255) ? "," : " ",
		   bytenum, bitstring(bytenum));
  }

  printf("};\n");

  exit(0);
}

char *
bitstring(int val)
{
  int bitnum;
  char *p;
  static char bitstr[10];

  p = bitstr;
  for (bitnum = 7; bitnum >= 0; bitnum--)
    *p++ = (val & (1 << bitnum)) ? '1' : '0';
  *p = '\0';

  return(bitstr);
}

