/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/*
 * List the keys in a database to stdout.
 * Usage: dxlist volume [volume ...]
 *
 * Where:
 *   volume [volume ...] are the volumes of the db
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>

#include "dx.h"

#define MAX_VOLUMES		100

char buf[10240];

int
main(int argc, char **argv)
{
  int i;
  Dx *dbm;
  Dx_rc rc;
  Dx_datum key, value;
  Dx_config config;
  char *volumes[MAX_VOLUMES];

  if (argc < 2) {
      fprintf(stderr, "Usage: dxlist volume [volume ...]\n");
      exit(1);
  }

  if ((argc - 1) >= MAX_VOLUMES) {
    fprintf(stderr, "dxlist: too many volumes\n");
    exit(1);
  }

  for (i = 1; i < argc; i++)
    volumes[i-1] = argv[i];
  volumes[i-1] = NULL;

  Dx_get_configuration(NULL, &config);
  config.flags |= (DX_CONFIG_DEFAULT_FLAGS | DX_CONFIG_READ_ONLY);

  if ((rc = Dx_open(volumes, &config, &dbm)) != DX_OK) {
    fprintf(stderr, "dxlist: %s\n", Dx_strerror(rc));
    exit(1);
  }

  Dx_key(&key, NULL, 0);
  for (rc = Dx_firstkeyx(dbm, &key, DX_PROBE, &value); rc == DX_OK;
	    rc = Dx_nextkeyx(dbm, &key, DX_PROBE, &value)) {
    printf("%-20s %8lu\n", (char *) key.dptr, value.dlen);
  }

  if (rc != DX_OK && rc != DX_KEY_NOT_FOUND) {
    fprintf(stderr, "dxlist: %s\n", Dx_strerror(rc));
    exit(1);
  }

  rc = Dx_close(dbm);

  exit(0);
}

