/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>

#include "/k/generic/include/gdbm/gdbm.h"

#include "dx.h"

char buf[102400];

int
main(int argc, char **argv)
{
	int nkeys;
	unsigned long nbytes;
	GDBM_FILE dbm;
	datum key, value;
	Dx *db;
	Dx_config config;
	Dx_datum outkey, outvalue;
	Dx_rc rc;
	char *volumes[2];
	void *xxx;

    if (argc != 3) {
        fprintf(stderr, "Usage: convgdbm gdbm-file dx-file\n");
        exit(1);
    }

	if ((dbm = gdbm_open(argv[1], 0, GDBM_READER, 0644, 0)) == NULL) {
		fprintf(stderr, "Can't open %s\n", argv[1]);
		exit(1);
	}

    Dx_get_configuration(NULL, &config);
    config.flags = DX_CONFIG_DEFAULT_FLAGS | DX_CONFIG_TRUNCATE_ON_OPEN;
	volumes[0] = argv[2];
	volumes[1] = NULL;
	if ((rc = Dx_open(volumes, &config, &db)) != DX_OK) {
		fprintf(stderr, "Db: %s\n", Dx_strerror(rc));
		exit(1);
	}

	nkeys = 0;
	nbytes = 0;
	xxx = NULL;
	for (key = gdbm_firstkey(dbm); key.dptr != NULL; key = gdbm_nextkey(dbm, key)) {
		value = gdbm_fetch(dbm, key);

		Dx_key(&outkey, key.dptr, key.dsize);
		Dx_value(&outvalue, value.dptr, value.dsize);
		if ((rc = Dx_store(db, &outkey, DX_INSERT, &outvalue)) != DX_OK) {
			fprintf(stderr, "Db: %s\n", Dx_strerror(rc));
			exit(1);
		}

		nkeys++;
		nbytes += key.dsize + value.dsize;
		free(value.dptr);
		if (xxx != NULL)
			free(xxx);
		xxx = key.dptr;
	}

	if (xxx != NULL)
		free(xxx);

	gdbm_close(dbm);

	if ((rc = Dx_close(db)) != DX_OK) {
		fprintf(stderr, "Db: %s\n", Dx_strerror(rc));
		exit(1);
	}

	fprintf(stderr, "%d keys, %lu total bytes\n", nkeys, nbytes);
	exit(0);
}

