/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>

#include "dx.h"

char buf[10240];

int
main(int argc, char **argv)
{
	int nkeys;
	unsigned long nbytes;
	Dx *dbm;
	Dx_rc rc;
	Dx_datum key, value;
	char *volumes[2];

    if (argc != 2) {
        fprintf(stderr, "Usage: countdx dx-file\n");
        exit(1);
    }

    volumes[0] = argv[1];
    volumes[1] = NULL;

	if ((rc = Dx_open(volumes, NULL, &dbm)) != DX_OK) {
		fprintf(stderr, "Dx: %s\n", Dx_strerror(rc));
		exit(1);
	}

	Dx_key(&key, NULL, 0);
	nkeys = 0;
	nbytes = 0;
	for (rc = Dx_firstkeyx(dbm, &key, DX_FETCH, &value);
			rc == DX_OK;
			rc = Dx_nextkeyx(dbm, &key, DX_FETCH, &value)) {
		nkeys++;
		nbytes += key.dlen + value.dlen;
	}

	if (rc != DX_OK && rc != DX_KEY_NOT_FOUND) {
	  fprintf(stderr, "Dx: %s\n", Dx_strerror(rc));
	  exit(1);
    }

	rc = Dx_close(dbm);

	fprintf(stderr, "%d keys, %lu total bytes\n", nkeys, nbytes);
	exit(0);
}

