/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>

#include "/k/generic/include/gdbm/gdbm.h"

char buf[10240];

int
main(int argc, char **argv)
{
	int nkeys;
	unsigned long nbytes;
	GDBM_FILE dbm;
	datum key, value;
	void *xxx;

    if (argc != 2) {
        fprintf(stderr, "Usage: countgdbm gdbm-file\n");
        exit(1);
    }

	if ((dbm = gdbm_open(argv[1], 0, GDBM_READER, 0644, 0)) == NULL) {
		fprintf(stderr, "Can't open %s\n", argv[1]);
		exit(1);
	}

	nkeys = 0;
	nbytes = 0;
	xxx = NULL;
	for (key = gdbm_firstkey(dbm); key.dptr != NULL;
			key = gdbm_nextkey(dbm, key)) {
		value = gdbm_fetch(dbm, key);

		nkeys++;
		nbytes += key.dsize + value.dsize;
		free(value.dptr);
		if (xxx != NULL)
			free(xxx);
		xxx = key.dptr;
	}

	if (xxx != NULL)
		free(xxx);

	gdbm_close(dbm);

	fprintf(stderr, "%d keys, %lu total bytes\n", nkeys, nbytes);
	exit(0);
}

