/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>

#include "sdbm.h"

char buf[10240];

int
main(int argc, char **argv)
{
	int nkeys;
	unsigned long nbytes;
	DBM *dbm;
	datum key, value;

    if (argc != 2) {
        fprintf(stderr, "Usage: countsdbm sdbm-file\n");
        exit(1);
    }

	if ((dbm = sdbm_open(argv[1], O_RDONLY, 0644)) == NULL) {
		fprintf(stderr, "Can't open %s\n", argv[1]);
		exit(1);
	}

	nkeys = 0;
	nbytes = 0;
	for (key = sdbm_firstkey(dbm); key.dptr != NULL; key = sdbm_nextkey(dbm)) {
		value = sdbm_fetch(dbm, key);

		nkeys++;
		nbytes += key.dsize + value.dsize;
	}

	sdbm_close(dbm);

	fprintf(stderr, "%d keys, %lu total bytes\n", nkeys, nbytes);
	exit(0);
}

