/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>

#include "dx.h"

char buf[102400];

static void del_words(Dx *dbm);

int
main(int argc, char **argv)
{
	Dx *dbm;
	Dx_config config;
	Dx_rc rc;
	char *volumes[2];

    if (argc != 2) {
        fprintf(stderr, "Usage: deldx dx-file\n");
        exit(1);
    }

    Dx_get_configuration(NULL, &config);
    config.flags = DX_CONFIG_DEFAULT_FLAGS;
	/*config.page_size = 32768;*/
    volumes[0] = argv[1];
    volumes[1] = NULL;
	if ((rc = Dx_open(volumes, &config, &dbm)) != DX_OK) {
	  fprintf(stderr, "Dx: %s\n", Dx_strerror(rc));
	  exit(1);
	}

    del_words(dbm);

	Dx_close(dbm);

	exit(0);
}

int nwords;

static void
del_words(Dx *dbm)
{
  int len, rc;
  Dx_datum key;

  nwords = 0;
  while (fgets(buf, sizeof(buf), stdin) != NULL) {
    len = strlen(buf);
    len--;
    buf[len] = '\0';
    Dx_key(&key, buf, len + 1);
    rc = Dx_delete(dbm, &key);
    if (rc != DX_OK) {
      fprintf(stderr, "Failed\n");
	  fprintf(stderr, "Dx: %s\n", Dx_strerror(rc));
      fprintf(stderr, "Key: %s\n", buf);
	  exit(1);
    }
    else
      nwords++;
  }
  fprintf(stderr, "Deleted %d items\n", nwords);
}

