/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/*
 * This is just a simple frame memory test/demonstration.
 *
 * With no command line arguments, it creates (or truncates) a database
 * called "db1" and stores some strings in it.
 *
 * When run with one or more command line arguments (it doesn't matter what
 * they are), it retrieves and prints those strings.
 */

#include <stdio.h>

#include "dx.h"

int
main(int argc, char **argv)
{
  int i;
  double *d;
  Dx *db, *db2;
  void *f, *fbuf;
  size_t buflen;
  Dx_config config;
  Dx_rc rc;
  Dx_datum key, value;
  char *volumes[2], **vec;

  volumes[0] = "db1";
  volumes[1] = NULL;
  Dx_get_configuration(NULL, &config);
  config.flags |= DX_CONFIG_ERASE_ON_DELETE;

  if (argc == 1)
	config.flags |= DX_CONFIG_TRUNCATE_ON_OPEN;

  if ((rc = Dx_open(volumes, &config, &db)) != DX_OK) {
	fprintf(stderr, "Dx: %s\n", Dx_strerror(rc));
	exit(1);
  }

  if (argc > 1) {
	Dx_key(&key, "mykey", 6);
	rc = Dx_fetch(db, &key, &value);
	if (rc != DX_OK) {
	  fprintf(stderr, "Dx: %s\n", Dx_strerror(rc));
	  exit(1);
	}
	Dx_frame_import(db, value.dptr, value.dlen, (void *) &vec);
	Dx_unswizzle(db, (void **) &vec[0]);
	Dx_unswizzle(db, (void **) &vec[1]);
	Dx_unswizzle(db, (void **) &vec[2]);
	Dx_unswizzle(db, (void **) &vec[3]);
	Dx_unswizzle(db, (void **) &vec[4]);
	for (i = 0; vec[i] != NULL; i++)
	  printf("%s\n", vec[i]);

	Dx_close(db);
	exit(0);
  }

  Dx_new_frame(db, "persistent frame");
  vec = (char **) Dx_malloc(10 * sizeof(char *));
  vec[0] = (char *) Dx_malloc(11);
  strcpy(vec[0], "abcdefghij");
  vec[1] = (char *) Dx_malloc(50);
  strcpy(vec[1], "Auggie+Harley");

  Dx_new_frame(db, "persistent frame2");
  vec[2] = (char *) Dx_malloc(33);
  strcpy(vec[2], "Hello world.");
  vec[3] = (char *) Dx_malloc(11);
  strcpy(vec[3], "0123456789");
  vec[4] = NULL;

  f = Dx_pop_frame();
  Dx_unite_frames(NULL, f);

  Dx_frame_export(db, vec, &fbuf, &buflen);
  Dx_swizzle(db, (void **) &vec[0]);
  Dx_swizzle(db, (void **) &vec[1]);
  Dx_swizzle(db, (void **) &vec[2]);
  Dx_swizzle(db, (void **) &vec[3]);
  Dx_swizzle(db, (void **) &vec[4]);

  Dx_key(&key, "mykey", 6);
  Dx_value(&value, fbuf, buflen);
  rc = Dx_store(db, &key, DX_INSERT_UNIQUE, &value);
  if (rc != DX_OK) {
	fprintf(stderr, "Dx: %s\n", Dx_strerror(rc));
	exit(1);
  }

  Dx_get_configuration(NULL, &config);
  config.flags |= DX_CONFIG_ERASE_ON_DELETE;
  if ((rc = Dx_open(NULL, &config, &db2)) != DX_OK) {
	fprintf(stderr, "Dx: %s\n", Dx_strerror(rc));
	exit(1);
  }
  Dx_new_frame(db2, "volatile frame");
  d = (double *) Dx_malloc(sizeof(double));
  *d = 3.1415;

  Dx_free();
  Dx_close(db);
  
  *d = 1.0;
  Dx_close(db2);

  exit(0);
}
