/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/*
 * Insert/Replace test
 *   no args
 */

#include <stdio.h>

#include "dx.h"

void fetchem(Dx *db);

int
main(int argc, char **argv)
{
  int i;
  char *volumes[10];
  Dx_datum key, value;
  Dx *db;
  Dx_config config;
  Dx_len spos, slen;
  Dx_opmode mode;
  Dx_point pos;
  Dx_user_io uio;
  Dx_rc rc;

  volumes[0] = "../db1";
  volumes[1] = "../db2";
  volumes[2] = "../db3";
  volumes[3] = NULL;
  Dx_get_configuration(NULL, &config);
  config.flags |= DX_CONFIG_TRUNCATE_ON_OPEN;

  if ((rc = Dx_open(volumes, &config, &db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  Dx_key(&key, "anykey", 7);
  Dx_value(&value, "value1", 7);
  if ((rc = Dx_storex(db, &key, DX_INSERT | DX_ADD_LAST, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }

  Dx_key(&key, "anykey", 7);
  Dx_value(&value, "value2", 7);
  if ((rc = Dx_storex(db, &key, DX_INSERT | DX_ADD_LAST, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }

  Dx_key(&key, "anykey", 7);
  Dx_value(&value, "value3", 7);
  if ((rc = Dx_storex(db, &key, DX_INSERT | DX_ADD_LAST, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }

  Dx_key(&key, "anykey", 7);
  Dx_value(&value, "value4", 7);
  if ((rc = Dx_storex(db, &key, DX_INSERT | DX_ADD_LAST, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }

  fetchem(db);

  Dx_key(&key, "anykey", 7);
  Dx_value(&value, "newvalue3", 10);
  key.ind = 2;
  if ((rc = Dx_storex(db, &key, DX_REPLACE, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }

  printf("\n");
  fetchem(db);

  if ((rc = Dx_close(db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  exit(0);
}

void
fetchem(Dx *db)
{
  int i;
  Dx_datum key, value;
  Dx_rc rc;

  for (i = 0; i < 4; i++) {
    if (i == 0) {
      if ((rc = Dx_firstkeyx(db, &key, DX_FETCH, &value)) != DX_OK) {
        fprintf(stderr, "%s\n", Dx_strerror(rc));
        exit(1);
      }
    }
    else {
      if ((rc = Dx_nextkeyx(db, &key, DX_FETCH, &value)) != DX_OK) {
        fprintf(stderr, "%s\n", Dx_strerror(rc));
        exit(1);
      }
    }
	printf("%d. %s\n", i, (char *) value.dptr);
  }
}
