/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/*
 * Check the keys in a one volume db.
 * Reads keys from stdin, one per line, and fetches the values.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>

#include "dx.h"

char buf[102400];

static void check_keys(Dx *dbm);
Dx_len io_putfunc(void *arg, void *dptr, Dx_len dlen, Dx_desc *desc);

int
main(int argc, char **argv)
{
	Dx *dbm;
	Dx_config config;
	Dx_rc rc;
	char *volumes[10];

    if (argc != 2) {
        fprintf(stderr, "Usage: checkdx dxfile\n");
        exit(1);
    }

    Dx_get_configuration(NULL, &config);
    config.flags = DX_CONFIG_DEFAULT_FLAGS;
	config.max_stripe_npages = 1024;
	/*config.page_size = 32768;*/
    volumes[0] = argv[1];
    volumes[1] = NULL;
	if ((rc = Dx_open(volumes, &config, &dbm)) != DX_OK) {
	  fprintf(stderr, "Dx: %s\n", Dx_strerror(rc));
	  exit(1);
	}

    check_keys(dbm);

	Dx_close(dbm);

	exit(0);
}

int nwords;

struct io_info {
  int fd;
  Dx_len count;
};

static void
check_keys(Dx *dbm)
{
  int len, rc;
  Dx_datum key;
  Dx_user_io uio;
  struct io_info io_info;

  nwords = 0;
  while (fgets(buf, sizeof(buf), stdin) != NULL) {
    len = strlen(buf);
    len--;
    buf[len] = '\0';
	Dx_key(&key, buf, len + 1);
    io_info.fd = 1;
    io_info.count = 0;
    uio.func = io_putfunc;
    uio.arg = (void *) &io_info;

    rc = Dx_fetchx(dbm, &key, DX_FETCH | DX_UIO, &uio);
    if (rc != DX_OK) {
      fprintf(stderr, "Failed\n");
      fprintf(stderr, "Key: %s\n", buf);
    }
    else
      nwords++;
  }
}

Dx_len
io_putfunc(void *arg, void *dptr, Dx_len dlen, Dx_desc *desc)
{
  int fd, n;
  struct io_info *info;

  info = (struct io_info *) arg;
  fd = info->fd;

  if (fd == -1)
    return(0);

  n = write(fd, dptr, (size_t) dlen);

  if (n != dlen) {
	desc->error_occurred = 1;
	desc->last_errno = errno;
  }

  if (n > 0)
    info->count += n;

  return((Dx_len) n);
}
