/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/*
 * Probe/Exists test
 *   no args
 */

#include <stdio.h>

#include "dx.h"

int
main(int argc, char **argv)
{
  int i;
  char *volumes[10], buf[10000];
  Dx_datum key, value;
  Dx *db;
  Dx_config config;
  Dx_len spos, slen;
  Dx_opmode mode;
  Dx_point pos;
  Dx_region region;
  Dx_user_io uio;
  Dx_rc rc;

  volumes[0] = "../db1";
  volumes[1] = "../db2";
  volumes[2] = NULL;
  Dx_get_configuration(NULL, &config);
  config.flags |= DX_CONFIG_TRUNCATE_ON_OPEN;

  if ((rc = Dx_open(volumes, &config, &db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  Dx_key(&key, "smallvalue", 11);
  Dx_value(&value, "abcdefghijklmnopqrstuvwxyz", 27);
  if ((rc = Dx_store(db, &key, DX_INSERT, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }

  Dx_key(&key, "smallvalue", 11);
  if ((rc = Dx_fetchx(db, &key, DX_EXISTS)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }
  printf("DX_EXISTS ok\n");

  Dx_key(&key, "smallvalue", 11);
  if ((rc = Dx_fetchx(db, &key, DX_PROBE, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }
  printf("(dlen=%lu, count=%u)\n", value.dlen, value.count);

  Dx_key(&key, "smallvalue", 11);
  if ((rc = Dx_fetchx(db, &key, DX_FETCH, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }
  printf("%s (dlen=%lu, count=%u)\n", (char *) value.dptr, value.dlen,
		value.count);

  Dx_set_region_start(&region, 10);
  Dx_set_region_len(&region, 10);
  Dx_key(&key, "smallvalue", 11);
  if ((rc = Dx_fetchx(db, &key, DX_EXISTS | DX_REGION, &region)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }
  printf("DX_EXISTS ok\n");

  Dx_key(&key, "smallvalue", 11);
  if ((rc = Dx_fetchx(db, &key, DX_PROBE | DX_REGION, &region, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }
  printf("(dlen=%lu, count=%u)\n", value.dlen, value.count);

  Dx_key(&key, "smallvalue", 11);
  if ((rc = Dx_fetchx(db, &key, DX_FETCH | DX_REGION, &region, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }
  printf("%.*s (dlen=%lu, count=%u)\n", (int) value.dlen, (char *) value.dptr,
		value.dlen, value.count);

  if ((rc = Dx_close(db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  exit(0);
}

