/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/*
 * Fetch test
 *   volume: the first (or only) volume
 *   [volume2] [volume3] ...
 *   arg: a key
 *   arg: the start position (0..LEN-1)
 *   arg: the length to extract (0..LEN | "all")
 */

#include <stdio.h>

#include "dx.h"

Dx_len
iofunc(void *arg, void *dptr, Dx_len dlen, Dx_desc *desc)
{
  ssize_t n;

  n = write(1, dptr, (size_t) dlen);
  if (n == -1) {
    desc->error_occurred = 1;
    desc->last_errno = 1;
    return(0);
  }
  return((Dx_len) n);
}

int
main(int argc, char **argv)
{
  int exitcode, i;
  char *volumes[10];
  Dx_datum key, value;
  Dx *db;
  Dx_config config;
  Dx_len slen, spos;
  Dx_opmode mode;
  Dx_region chunk;
  Dx_user_io uio;
  Dx_rc rc;

  if (argc < 5) {
    fprintf(stderr, "Usage: test02 volume [volume ...] key startpos length\n");
    exit(1);
  }
  exitcode = 0;

  for (i = 1; i < (argc - 3); i++)
    volumes[i - 1] = argv[i];
  volumes[i - 1] = NULL;

  Dx_get_configuration(NULL, &config);
  config.flags |= (DX_CONFIG_DEFAULT_FLAGS | DX_CONFIG_READ_ONLY);

  if ((rc = Dx_open(volumes, &config, &db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  Dx_key(&key, argv[i], strlen(argv[i]) + 1);
  i++;

  spos = (Dx_len) strtoul(argv[i], NULL, 10);
  Dx_set_region_start(&chunk, spos);
  i++;

  if (strcmp(argv[i], "all") == 0)
    Dx_set_region_to_end(&chunk);
  else {
    slen = (Dx_len) strtoul(argv[i], NULL, 10);
    if (spos < 0)
      Dx_set_region_from_end(&chunk, -spos);
    else
      Dx_set_region_len(&chunk, slen);
  }

  if (argv[i] != NULL) {
    fprintf(stderr, "Usage: test02 volume [volume ...] key startpos length\n");
    exitcode = 1;
    goto err;
  }

  uio.func = iofunc;
  uio.arg = (void *) NULL;
  if ((rc = Dx_fetchx(db, &key, DX_FETCH | DX_UIO | DX_REGION,
					  &chunk, &uio)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exitcode = 1;
    goto err;
  }

 err:

  if ((rc = Dx_close(db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  exit(exitcode);
}

