/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/*
 * Store test - overwrite part of direct datum
 *   arg1: a key
 *   arg2: the start position (1..LEN)
 *   arg3: the length to overwrite (0..LEN)
 */

#include <stdio.h>

#include "dx.h"

char buf[10240];

Dx_len
iofunc(void *arg, void *dptr, Dx_len dlen, Dx_desc *desc)
{
  int x;

  x = (int) arg;
  fprintf(stderr, "`%.*s' (%ld)\n", (int) dlen, (char *) dptr, dlen);
  return(dlen);
}

int
main(int argc, char **argv)
{
  int i;
  char *volumes[10];
  Dx_datum key, value;
  Dx *db;
  Dx_config config;
  Dx_len spos, slen;
  Dx_opmode mode;
  Dx_point pos;
  Dx_user_io uio;
  Dx_rc rc;

  volumes[0] = "../db1";
  volumes[1] = "../db2";
  volumes[2] = "../db3";
  volumes[3] = NULL;
  Dx_get_configuration(NULL, &config);
  config.flags |= DX_CONFIG_DEFAULT_FLAGS;

  if ((rc = Dx_open(volumes, &config, &db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  Dx_key(&key, argv[1], strlen(argv[1]) + 1);

  spos = atoi(argv[2]) - 1;
  slen = atoi(argv[3]);

  Dx_set_point(&pos, spos);

  for (i = 0; i < slen; i++)
	buf[i] = 'x';
  Dx_value(&value, buf, slen);

  if ((rc = Dx_storex(db, &key, DX_STORE | DX_OVERWRITE, &pos, &value))
	  != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }

  if ((rc = Dx_fetchx(db, &key, DX_FETCH, &value)) != DX_OK) {
    fprintf(stderr, "%s\n", Dx_strerror(rc));
    exit(1);
  }

  fprintf(stderr, "`%.*s' (%lu bytes, desc=%u)\n",
		  (int) value.dlen, (char *) value.dptr,
		  (unsigned long) value.dlen, value.flags);

  if ((rc = Dx_close(db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  exit(0);
}

