/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/*
 * Insert test
 * arg 1: key
 * arg 2: insert position
 * arg 3: file name (new data)
 */

#include <errno.h>
#include <stdio.h>

#include "dx.h"

FILE *fp;

Dx_len
iofunc(void *arg, void *dptr, Dx_len dlen, Dx_desc *desc)
{
  size_t n;

  if (fp == NULL)
	return(0);

  n = fread(dptr, sizeof(char), (size_t) dlen, fp);
  if (ferror(fp)) {
    desc->error_occurred = 1;
    desc->last_errno = errno;
  }
  if (feof(fp)) {
	fclose(fp);
	fp = NULL;
  }

  return((Dx_len) n);
}

int
main(int argc, char **argv)
{
  int do_edit, do_insert, do_io, i;
  char *volumes[10];
  Dx_datum key, value;
  Dx *db;
  Dx_config config;
  Dx_opmode mode;
  Dx_point pos;
  Dx_region edit;
  Dx_user_io uio;
  Dx_rc rc;

  volumes[0] = "../db1";
  volumes[1] = "../db2";
  volumes[2] = "../db3";
  volumes[3] = NULL;
  Dx_get_configuration(NULL, &config);
  config.flags |= DX_CONFIG_DEFAULT_FLAGS;

  if ((rc = Dx_open(volumes, &config, &db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  Dx_key(&key, argv[1], strlen(argv[1]) + 1);

  if (strcmp(argv[2], "-e") == 0)
	Dx_set_point_at_end(&pos);
  else if (isdigit(argv[2][0]))
	Dx_set_point(&pos, (Dx_len) atol(argv[2]));

  fp = fopen(argv[3], "r");
  if (fp == NULL) {
    perror("fopen");
    exit(1);
  }

  uio.func = iofunc;
  uio.arg = NULL;

  rc = Dx_storex(db, &key, DX_STORE | DX_UIO | DX_MAY_CREATE, &pos, &uio);

  if (rc != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  rc = Dx_fetch(db, &key, &value);

  if (rc != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  if ((rc = Dx_close(db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  exit(0);
}

