/*
 * Copyright (c) 2000 DSS Distributed Systems Software, Inc.
 * All rights reserved.
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/*
 * Edit (delete & insert) test
 *  arg 1: key
 *  arg 2: flag
 *  arg 3: edit start position
 *  arg 4: edit length
 *  arg 5: file name (new data)
 */

#include <errno.h>
#include <stdio.h>

#include "dx.h"

FILE *fp;

Dx_len
iofunc(void *arg, void *dptr, Dx_len dlen, Dx_desc *desc)
{
  size_t n;

  n = fread(dptr, sizeof(char), dlen, fp);
  if (ferror(fp)) {
    desc->error_occurred = 1;
    desc->last_errno = errno;
  }
  return((Dx_len) n);
}

int
main(int argc, char **argv)
{
  int do_edit, do_insert, do_io, i;
  char *volumes[10];
  Dx_datum key, value;
  Dx *db;
  Dx_config config;
  Dx_opmode mode;
  Dx_len epos, slen, spos;
  Dx_region edit;
  Dx_point chunk;
  Dx_user_io uio;
  Dx_rc rc;

  volumes[0] = "../db1";
  volumes[1] = "../db2";
  volumes[2] = "../db3";
  volumes[3] = NULL;

  if ((rc = Dx_open(volumes, NULL, &db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  Dx_key(&key, argv[1], strlen(argv[1]) + 1);

  if (strcmp(argv[2], "-pl") == 0) {
	spos = atoi(argv[3]);
	slen = atoi(argv[4]);
	Dx_set_region_start(&edit, spos);
	Dx_set_region_len(&edit, slen);
	fp = fopen(argv[5], "r");
  }
  else if (strcmp(argv[2], "-pe") == 0) {
	spos = atoi(argv[3]);
	Dx_set_region_start(&edit, spos);
	Dx_set_region_to_end(&edit);
	fp = fopen(argv[4], "r");
  }
  else if (strcmp(argv[2], "-pp") == 0) {
	spos = atoi(argv[3]);
	epos = atoi(argv[4]);
	Dx_set_region_start(&edit, spos);
	Dx_set_region_end(&edit, epos);
	fp = fopen(argv[5], "r");
  }
  else if (strcmp(argv[2], "-all") == 0) {
	Dx_set_region_from_start(&edit);
	Dx_set_region_to_end(&edit);
	fp = fopen(argv[3], "r");
  }

  if (fp == NULL) {
    perror("fopen");
    exit(1);
  }


  uio.func = iofunc;
  uio.arg = NULL;

  rc = Dx_storex(db, &key, DX_EDIT | DX_UIO, &edit, &uio);

  if (rc != DX_OK) {
	  fprintf(stderr, "%s\n", Dx_strerror(rc));
	  exit(1);
  }

  if ((rc = Dx_close(db)) != DX_OK) {
	fprintf(stderr, "%s\n", Dx_strerror(rc));
	exit(1);
  }

  exit(0);
}

