
#include "php.h"

#if DBA_DX
#include "php3_dx.h"

/*#include <dx.h>*/
#include "/tmp/dx/dx.h"

DBA_OPEN_FUNC(dx)
{
	char *volumes[2];
	Dx *dbf;
	Dx_config config;
	Dx_rc rc;
	dba_info *pinfo = (dba_info *) info;

	Dx_get_configuration(NULL, &config);

	switch(info->mode) {
	case DBA_READER:
		config.flags |= DX_CONFIG_READ_ONLY;
		break;
	case DBA_WRITER:
		config.flags |= DX_CONFIG_MUST_EXIST;
		break;
	case DBA_CREAT:
		/* This is the default. */
		break;
	case DBA_TRUNC:
		config.flags |= DX_CONFIG_TRUNCATE_ON_OPEN;
		break;
	default:
		return FAILURE;
		/*NOTREACHED*/
	}

	if(info->argc > 0) {
		convert_to_long(info->argv[0]);
		config.open_mode = (mode_t) info->argv[0]->value.lval;
	}

	volumes[0] = info->path;
	volumes[1] = NULL;
	rc = Dx_open(volumes, &config, &dbf);
	
	if(rc == DX_OK) {
		pinfo->dbf = dbf;
		return SUCCESS;
	}
	pinfo->dbf = NULL;
	return FAILURE;
}

DBA_CLOSE_FUNC(dx)
{
	
	Dx_close(info->dbf);
}

DBA_FETCH_FUNC(dx)
{
	Dx_datum dxkey, dxvalue;
	char *new = NULL;
	Dx_rc rc;

	Dx_key(&dxkey, key, keylen);
	rc = Dx_fetch(info->dbf, &dxkey, &dxvalue);
	if(rc == DX_OK) {
		if(newlen) *newlen = (unsigned int) dxvalue.dlen;
		new = estrndup(dxvalue.dptr, (unsigned int) dxvalue.dlen);
		if (dxvalue.flags & DX_DATUM_MALLOC)
			free(dxvalue.dptr);
	}
	return new;
}

DBA_UPDATE_FUNC(dx)
{
	Dx_datum dxkey, dxvalue;
	Dx_rc rc;

	Dx_key(&dxkey, key, keylen);
	Dx_value(&dxvalue, val, vallen);

	rc = Dx_store(info->dbf, &dxkey, mode == 1 ? DX_INSERT : DX_REPLACE,
			&dxvalue);
	if(rc == DX_OK)
		return SUCCESS;
	printf("XXX %s\n", Dx_strerror(rc));
	return FAILURE;
}

DBA_EXISTS_FUNC(dx)
{
	Dx_datum dxkey;

	Dx_key(&dxkey, key, keylen);
	return ((Dx_fetchx(info->dbf, &dxkey, DX_EXISTS) == DX_OK)
		? SUCCESS : FAILURE);
}

DBA_DELETE_FUNC(dx)
{
	Dx_datum dxkey;
	
	Dx_key(&dxkey, key, keylen);
	return ((Dx_delete(info->dbf, &dxkey) == DX_OK)
		? SUCCESS : FAILURE);
}

DBA_FIRSTKEY_FUNC(dx)
{
	Dx_datum dxkey;
	char *key = NULL;
	Dx_rc rc;

	Dx_key(&dxkey, NULL, 0);
	rc = Dx_firstkey(info->dbf, &dxkey);
	if (rc == DX_OK) {
		key = estrndup(dxkey.dptr, (unsigned int) dxkey.dlen);
		if(newlen) *newlen = (unsigned int) dxkey.dlen;
		if(dxkey.flags & DX_DATUM_MALLOC)
			free(dxkey.dptr);
	}

	return key;
}

DBA_NEXTKEY_FUNC(dx)
{
	char *nkey = NULL;
	Dx_datum dxkey;
	Dx_rc rc;

	Dx_key(&dxkey, NULL, 0);
	rc = Dx_nextkey(info->dbf, &dxkey);
	if(rc == DX_OK) {
		nkey = estrndup(dxkey.dptr, (unsigned int) dxkey.dlen);
		if(newlen) *newlen = (unsigned int) dxkey.dlen;
		if(dxkey.flags & DX_DATUM_MALLOC)
			free(dxkey.dptr);
	}

	return nkey;
}

DBA_OPTIMIZE_FUNC(dx)
{
	Dx_rc rc;

	rc = Dx_reorganize(info->dbf);
	return (rc == DX_OK ? SUCCESS : FAILURE);
}

DBA_SYNC_FUNC(dx) 
{
	Dx_rc rc;

	rc = Dx_sync(info->dbf);
	return (rc == DX_OK ? SUCCESS : FAILURE);
}
#endif
